'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _server = require('react-dom/server');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function stringify(obj) {
    return (0, _stringify2.default)(obj, (key, value) => {
        if (value instanceof Function || typeof value === 'function') {
            return value.toString();
        }
        if (value instanceof RegExp) {
            return `_PxEgEr_${ value }`;
        }

        return value;
    });
}

class FluxController {
    constructor(appProperties) {
        if (!appProperties) {
            appProperties = {};
        }

        if (typeof window !== 'undefined' && window.fluxAppProperties) {
            appProperties = window.fluxAppProperties;
        }

        this.appProperties = _immutable2.default.fromJS(appProperties);

        this.fluxContext = {};
        this.stores = {};
        this.eventEmitter = new _events2.default.EventEmitter();
    }

    getStore(key) {
        return this.stores[key];
    }

    removeStore(key) {
        delete this.stores[key];
    }

    createStore(StoreClass, key) {
        let store = this.stores[key];
        if (!store) {
            for (var _len = arguments.length, params = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
                params[_key - 2] = arguments[_key];
            }

            store = this.stores[key] = new StoreClass(key, this.fluxContext, this, ...params);
        }

        return store;
    }

    renderToString() {
        const appProperties = stringify(this.appProperties.toJS());
        const storesData = stringify(_lodash2.default.map(this.stores, store => store._render()));

        const script = `
            function parse(str, date2obj) {
                var iso8061 = date2obj ? /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/ : false;

                return JSON.parse(str, function (key, value) {
                  var prefix;

                  if (typeof value != 'string') {
                    return value;
                  }
                  if (value.length < 8) {
                    return value;
                  }

                  prefix = value.substring(0, 8);

                  if (iso8061 && value.match(iso8061)) {
                    return new Date(value);
                  }
                  if (prefix === 'function') {
                    return eval('(' + value + ')');
                  }
                  if (prefix === '_PxEgEr_') {
                    return eval(value.slice(8));
                  }

                  return value;
                });
            }

            window.fluxAppProperties = parse(${ (0, _stringify2.default)(appProperties) });
            window.fluxStores = parse(${ (0, _stringify2.default)(storesData) });
        `;

        const markup = _react2.default.createElement('script', { type: 'text/javascript', dangerouslySetInnerHTML: { __html: script } });

        return (0, _server.renderToString)(markup);
    }
}
exports.default = FluxController;